/* need to include STDIO.H for FPRINTF() */

#define SUCCESS 0
#define FAILURE 1

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif
static int err_mac_ans;

/*
 * For functions returning int, when you want an immediate return.
 */

/* CHK() fails if (a != SUCCESS) */
#define CHK(a)                                                        \
  do {                                                                \
    if( (err_mac_ans = (a)) != SUCCESS ) {                            \
      fprintf(stderr,"FAILURE: %s.%d. (%s) returned %d\n",__FILE__,__LINE__, #a, err_mac_ans); \
      fflush(stderr);                                                 \
      return err_mac_ans;                                             \
    }                                                                 \
  } while (0)

/* AST() fails if (a != 0) */
#define AST(a)                                                        \
  do {                                                                \
    if( (err_mac_ans = (a)) == FALSE ) {                              \
      fprintf(stderr, "ASSERT FAILED: %s.%d (%s) returned %d\n",__FILE__,__LINE__, #a, err_mac_ans); \
      fflush(stderr);                                                 \
      return FAILURE;                                                 \
    }                                                                 \
  } while(0)

/* like AST(), but return E_CODE instead of the default FAILURE */
#define E_AST(a,e_code)                                               \
  do {                                                                \
    if((a) == FALSE) {                                                \
      fprintf(stderr,"ASSERT FAILED: %s.%d (%s)\n",__FILE__,__LINE__, #a); \
      fflush(stderr);                                                 \
      return (e_code);                                                \
    }                                                                 \
  } while(0)

/* like E_AST(), but also fprintfs __VA_ARGS__ */
#define M_AST(a,e_code,...)                                           \
  do {                                                                \
    if((a) == FALSE) {                                                \
      fprintf(stderr,"ASSERT FAILED: %s.%d (%s) ",__FILE__,__LINE__, #a); \
      fprintf(stderr, __VA_ARGS__);                                   \
      fflush(stderr);                                                 \
      return (e_code);                                                \
    }                                                                 \
  } while(0)

/* print a message and return E_CODE */
#define M_EXIT(e_code, ... )                               \
  do {                                                     \
    fprintf(stderr,"EXIT: %s.%d ",__FILE__,__LINE__);      \
    fprintf(stderr, __VA_ARGS__ );                         \
    fflush(stderr);                                        \
    return (e_code);                                       \
  } while(0)

/*
 * When you don't want an immedate return, but would rather 'goto Error;'.
 * These set the local variable RC with the return value before jumping.
 */

#define G_CHK(a)                                                      \
  do {                                                                \
    if( (rc = (a)) != SUCCESS ) {                                     \
      fprintf(stderr,"FAILURE: %s.%d. (%s)\n",__FILE__,__LINE__, #a); \
      fflush(stderr);                                                 \
      goto Error;                                                     \
    }                                                                 \
  } while (0)

#define G_AST(a)                                                      \
  do {                                                                \
    if( (rc = (a)) == FALSE ) {                                       \
      fprintf(stderr, "ASSERT FAILED: %s.%d (%s)\n",__FILE__,__LINE__, #a); \
      fflush(stderr);                                                 \
      goto Error;                                                     \
    }                                                                 \
  } while(0)

#define G_E_AST(a,e_code)                                             \
  do {                                                                \
    if((a) == FALSE) {                                                \
      fprintf(stderr,"ASSERT FAILED: %s.%d (%s)\n",__FILE__,__LINE__, #a); \
      fflush(stderr);                                                 \
      rc = e_code;                                                    \
      goto Error;                                                     \
    }                                                                 \
  } while(0)

#define G_M_AST(a,e_code,...)                                         \
  do {                                                                \
    if((a) == FALSE) {                                                \
      fprintf(stderr,"ASSERT FAILED: %s.%d (%s) ",__FILE__,__LINE__, #a); \
      fprintf(stderr, __VA_ARGS__);                                   \
      fflush(stderr);                                                 \
      rc = e_code;                                                    \
      goto Error;                                                     \
    }                                                                 \
  } while(0)

/* print a message and return E_CODE */
#define G_M_EXIT(e_code, ... )                             \
  do {                                                     \
    fprintf(stderr,"EXIT: %s.%d ",__FILE__,__LINE__);      \
    fprintf(stderr, __VA_ARGS__ );                         \
    fflush(stderr);                                        \
    rc = e_code;                                           \
    goto Error;                                            \
  } while(0)

/*
 * And when you only want to print a message
 */
#define P_CHK(a)                                                      \
  do {                                                                \
    if( (a) != SUCCESS ) {                                            \
      fprintf(stderr,"FAILURE: %s.%d. (%s)\n",__FILE__,__LINE__, #a); \
      fflush(stderr);                                                 \
    }                                                                 \
  } while (0)

/* AST() fails if (a != 0) */
#define P_AST(a)                                                      \
  do {                                                                \
    if( (a) == FALSE ) {                                              \
      fprintf(stderr, "ASSERT FAILED: %s.%d (%s)\n",__FILE__,__LINE__, #a); \
      fflush(stderr);                                                 \
    }                                                                 \
  } while(0)
